#include <avr/io.h>      // this contains all the IO port definitions
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include <util/delay.h>
#include <avr/pgmspace.h> 

void delay_ms( uint16_t milliseconds)
{
   for( ; milliseconds > 0; milliseconds--)
   {
      _delay_ms( 1);
   }
} 

// http://www.dragonsgate.net/pipermail/icc-avr/2005-January/004853.html
#define M 0x7FFFFFFF  // 2^31-1, the modulus used by the psuedo-random
                      //  number generator prng().
uint32_t prng(uint32_t x)  // a good random number generator; call with 1 <= x <= M-1
{
	x = (x >> 16) + ((x << 15) & M) - (x >> 21) - ((x << 10) & M);
	if (x < 0) x += M;
	return x;
}


#define TIMER1_PRESCALE_1 1
#define TIMER1_PRESCALE_8 2
#define TIMER1_PRESCALE_64 3
#define TIMER1_PRESCALE_256 4
#define TIMER1_PRESCALE_1024 5

// We use these macros because binary constants arent always supported. ugh.
#define HEX__(n) 0x##n##UL
#define B8__(x) ((x&0x0000000FLU)?1:0)  \
               +((x&0x000000F0LU)?2:0)  \
               +((x&0x00000F00LU)?4:0)  \
               +((x&0x0000F000LU)?8:0)  \
               +((x&0x000F0000LU)?16:0) \
               +((x&0x00F00000LU)?32:0) \
               +((x&0x0F000000LU)?64:0) \
               +((x&0xF0000000LU)?128:0)
/*
#define B8__(x) ((x&0x0000000FLU)?128:0)  \
               +((x&0x000000F0LU)?64:0)  \
               +((x&0x00000F00LU)?32:0)  \
               +((x&0x0000F000LU)?16:0)  \
               +((x&0x000F0000LU)?8:0) \
               +((x&0x00F00000LU)?4:0) \
               +((x&0x0F000000LU)?2:0) \
               +((x&0xF0000000LU)?1:0)
//*/
#define B8(d) ((unsigned char)B8__(HEX__(d)))

const static uint8_t image[] PROGMEM = {

#include "./imagedata.c"

};

#define NUM_ELEM(x) (sizeof (x) / sizeof (*(x)))
uint8_t imagesize = NUM_ELEM(image);
uint8_t pausesize = 50;

// this function is called when timer1 compare matches OCR1A
volatile uint8_t j = 0;

SIGNAL( SIG_TIMER1_COMPA )
{
	if(j > (imagesize + pausesize)*2) 
		j = 0;


	if(j < imagesize)
		PORTB = pgm_read_byte(&(image[j]));
	else if(j > (pausesize + imagesize) && j <= (pausesize + imagesize*2))
		PORTB = pgm_read_byte(&(image[(pausesize + imagesize*2)-j]));
	else
		PORTB = 0x00;

	j++;
}

int main(void) {

  DDRB = 0xFF;       // set all 8 pins on port B to outputs

  /*
    the frequency of the interrupt overflow is determined by the 
    prescaler and overflow value.
    freq = clock_frequency / ( 2 * prescaler * overflow_val)
    where prescaler can be 1, 8, 64, 256, or 1024
    clock_freq is 8MHz
    and overflow_val is 16bit

    the overflow value is placed in OCR1A, the prescale is set in TCCR1B
    so for example:
    A good POV frequency is around 400Hz
    desired freq = 400Hz
    clock freq = 8MHz
    8MHz / (400Hz * 2) = 10000
    since 10000 is less than 655536 (largest 16 bit number)
    OCR1A = 10000 and the prescale is 1
  */

  TCCR1B = (1 << WGM12) | TIMER1_PRESCALE_1;
//  OCR1A = (uint16_t)10000;
  OCR1A = (uint16_t)8000;

  TIMSK |= 1 << OCIE1A;   // Output Compare Interrupt Enable (timer 1, OCR1A) 

  sei();                 // Set Enable Interrupts

  while (1);
}
